@@ -1,14 +1,19 @@
-#' @rdname textmodel_seededlda
+#' Unsupervised Latent Dirichlet allocation
+#'
+#' `textmodel_lda()` implements unsupervised Latent Dirichlet allocation (LDA).
+#' The estimator's code adopted from the GibbsLDA++ library (Phan & Nguyen,
+#' 2007).
+#'
 #' @param x the dfm on which the model will be fit
-#' @param k the number of topics; determined automatically by the number of keys
-#'   in `dictionary` in `textmodel_seededlda()`.
+#' @param k the number of topics.
 #' @param max_iter the maximum number of iteration in Gibbs sampling.
 #' @param verbose logical; if `TRUE` print diagnostic information during
 #'   fitting.
-#' @param alpha the value to smooth topic-document distribution; defaults to
-#'   `alpha = 50 / k`.
-#' @param beta the value to smooth topic-word distribution; defaults to `beta =
-#'   0.1`.
+#' @param alpha the value to smooth topic-document distribution.
+#' @param beta the value to smooth topic-word distribution.
+#' @param gamma a parameter to determine change of topics between sentences or
+#'   paragraphs. When `gamma > 0`, Gibbs sampling of topics for the current
+#'   document is affected by the previous document's topics.
 #' @param model a fitted LDA model; if provided, `textmodel_lda()` inherits
 #'   parameters from an existing model. See details.
 #' @details To predict topics of new documents (i.e. out-of-sample), first,
@@ -25,8 +30,26 @@
 #' @keywords textmodel
 #' @seealso [topicmodels][topicmodels::LDA]
 #' @export
+#' @examples
+#' \donttest{
+#' require(seededlda)
+#' require(quanteda)
+#'
+#' corp <- head(data_corpus_moviereviews, 500)
+#' toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE, remove_number = TRUE)
+#' dfmt <- dfm(toks) %>%
+#'     dfm_remove(stopwords('en'), min_nchar = 2) %>%
+#'     dfm_trim(min_termfreq = 0.90, termfreq_type = "quantile",
+#'              max_docfreq = 0.1, docfreq_type = "prop")
+#'
+#' lda <- textmodel_lda(head(dfmt, 450), 6)
+#' terms(lda)
+#' topics(lda)
+#' lda2 <- textmodel_lda(tail(dfmt, 50), model = lda) # new documents
+#' topics(lda2)
+#' }
 textmodel_lda <- function(
-    x, k = 10, max_iter = 2000, alpha = NULL, beta = NULL,
+    x, k = 10, max_iter = 2000, alpha = 0.5, beta = 0.1, gamma = 0,
     model = NULL, verbose = quanteda_options("verbose")
 ) {
     UseMethod("textmodel_lda")
@@ -34,7 +57,7 @@ textmodel_lda <- function(
 
 #' @export
 textmodel_lda.dfm <- function(
-    x, k = 10, max_iter = 2000, alpha = NULL, beta = NULL,
+    x, k = 10, max_iter = 2000, alpha = 0.5, beta = 0.1, gamma = 0,
     model = NULL, verbose = quanteda_options("verbose")
 ) {
 
@@ -46,45 +69,52 @@ textmodel_lda.dfm <- function(
         label <- rownames(model$phi)
         alpha <- model$alpha
         beta <- model$beta
+        if (!is.null(model$gamma)) {
+            gamma <- model$gamma
+        } else {
+            gamma <- 0
+        }
         words <- model$words
         warning("k, alpha and beta values are overwriten by the fitted model", call. = FALSE)
     } else {
         label <- paste0("topic", seq_len(k))
         words <- NULL
     }
-    lda(x, k, label, max_iter, alpha, beta, NULL, words, verbose)
+    lda(x, k, label, max_iter, alpha, beta, gamma, NULL, words, verbose)
 }
 
+is.textmodel_lda <- function(x) {
+    "textmodel_lda" %in% class(x)
+}
+
+
 #' @importFrom methods as
 #' @import quanteda
 #' @useDynLib seededlda, .registration = TRUE
-lda <- function(x, k, label, max_iter, alpha, beta, seeds, words, verbose) {
+lda <- function(x, k, label, max_iter, alpha, beta, gamma, seeds, words, verbose) {
 
-    k <- as.integer(k)
-    max_iter <- as.integer(max_iter)
-    if (is.null(alpha)) {
-        alpha <- -1.0 # default value will be set in C++
-    } else {
-        alpha <- as.double(alpha)
-    }
-    if (is.null(beta)) {
-        beta <- -1.0 # default value will be set in C++
-    } else {
-        beta <- as.double(beta)
-    }
-    verbose <- as.logical(verbose)
-
-    if (k < 1)
-        stop("k must be larger than zero")
+    k <- check_integer(k, min = 1, max = 1000)
+    alpha <- check_double(alpha, min = 0)
+    beta <- check_double(beta, min = 0)
+    gamma <- check_double(gamma, min = 0, max = 1)
+    verbose <- check_logical(verbose)
+    max_iter <- check_integer(max_iter)
 
     if (is.null(seeds))
-        seeds <- as(Matrix::Matrix(0, nrow = nfeat(x), ncol = k), "dgCMatrix")
+        seeds <- Matrix::Matrix(0, nrow = nfeat(x), ncol = k)
     if (is.null(words))
-        words <- as(Matrix::Matrix(0, nrow = nfeat(x), ncol = k), "dgCMatrix")
+        words <- Matrix::Matrix(0, nrow = nfeat(x), ncol = k)
+
+    first <- !duplicated(docid(x))
+    if (all(first) && gamma)
+        warning("gamma has no effect when docid are all unique.", call. = FALSE, immediate. = TRUE)
 
     random <- sample.int(.Machine$integer.max, 1) # seed for random number generation
-    result <- cpp_lda(x, k, max_iter, alpha, beta, seeds, words, random, verbose)
+    result <- cpp_lda(x, k, max_iter, alpha, beta, gamma,
+                      as(seeds, "dgCMatrix"), as(words, "dgCMatrix"),
+                      first, random, verbose)
 
+    dimnames(result$words) <- list(colnames(x), label)
     dimnames(result$phi) <- list(label, colnames(x))
     dimnames(result$theta) <- list(rownames(x), label)
     result$data <- x
@@ -93,7 +123,3 @@ lda <- function(x, k, label, max_iter, alpha, beta, seeds, words, verbose) {
     class(result) <- c("textmodel_lda", "textmodel", "list")
     return(result)
 }
-
-is.textmodel_lda <- function(x) {
-    "textmodel_lda" %in% class(x)
-}
