@@ -1,5 +1,7 @@
 require(quanteda)
 
+options(slda_residual_name = "other")
+
 toks <- tokens(data_corpus_moviereviews[1:500],
                remove_punct = TRUE,
                remove_symbols = TRUE,
@@ -11,9 +13,15 @@ sifi <- c("space", "mars", "alien", "earth")
 
 test_that("LDA is working", {
 
+    skip_on_os("mac")
+
     set.seed(1234)
     lda <- textmodel_lda(dfmt, k = 5)
-    # saveRDS(lda, "tests/data/lda_v081.RDS")
+    # saveRDS(lda, "tests/data/lda.RDS")
+
+    lda_v081 <- readRDS("../data/lda_v081.RDS")
+    expect_equal(lda$phi, lda_v081$phi)
+    expect_equal(lda$theta, lda_v081$theta)
 
     expect_equal(dim(terms(lda, 10)), c(10, 5))
     expect_equal(dim(terms(lda, 20)), c(20, 5))
@@ -22,7 +30,7 @@ test_that("LDA is working", {
         c("topic1", "topic2", "topic3", "topic4", "topic5")
     )
     expect_true(
-        sum(apply(terms(lda), 2, function(x)  all(sifi %in% x))) == 1 # there is a sifi topic
+        sum(apply(terms(lda), 2, function(x)  all(sifi %in% x))) == 1 # there is the sifi topic
     )
     expect_equal(
         names(topics(lda)),
@@ -52,8 +60,9 @@ test_that("LDA is working", {
     )
     expect_error(
         textmodel_lda(dfmt, k = 0),
-        "k must be larger than zero"
+        "The value of k must be between 1 and 1000"
     )
+
     expect_output(
         print(lda),
         paste0("\nCall:\n",
@@ -63,10 +72,34 @@ test_that("LDA is working", {
     )
     expect_equal(
         names(lda),
-        c("k", "max_iter", "last_iter", "alpha", "beta", "phi", "theta",
+        c("k", "max_iter", "last_iter", "alpha", "beta", "gamma", "phi", "theta",
           "words", "data", "call", "version")
     )
     expect_equivalent(class(lda$words), "dgCMatrix")
+    expect_equal(rownames(lda$words), colnames(lda$phi))
+    expect_equal(colnames(lda$words), rownames(lda$phi))
+})
+
+test_that("alpha and beta work", {
+
+    lda <- textmodel_lda(dfmt, max_iter = 100)
+    expect_equal(lda$alpha, 0.5)
+    expect_equal(lda$beta, 0.1)
+
+    lda2 <- textmodel_lda(dfmt, alpha = 0.7, beta = 0.2, max_iter = 100)
+    expect_equal(lda2$alpha, 0.7)
+    expect_equal(lda2$beta, 0.2)
+
+    expect_error(
+        textmodel_lda(dfmt, alpha = -0.1),
+        "The value of alpha must be between 0 and Inf"
+    )
+
+    expect_error(
+        textmodel_lda(dfmt, beta = -0.1),
+        "The value of beta must be between 0 and Inf"
+    )
+
 })
 
 test_that("verbose works", {
@@ -96,36 +129,6 @@ test_that("LDA works with empty documents", {
 
 })
 
-test_that("predict works with LDA", {
-    skip_on_cran()
-
-    dfmt_train <- head(dfmt, 450)
-    dfmt_test <- tail(dfmt, 50)
-
-    lda <- textmodel_lda(dfmt_train, k = 5)
-
-    expect_warning({
-        pred_train <- predict(lda)
-    })
-    expect_equal(names(pred_train), docnames(dfmt_train))
-    expect_equal(
-        levels(pred_train),
-        c("topic1", "topic2", "topic3", "topic4", "topic5")
-    )
-    expect_true(sum(topics(lda) == pred_train) / length(pred_train) > 0.9)
-
-    expect_warning({
-        pred_test <- predict(lda, newdata = dfmt_test)
-    })
-    expect_equal(names(pred_test), docnames(dfmt_test))
-    expect_equal(
-        levels(pred_test),
-        c("topic1", "topic2", "topic3", "topic4", "topic5")
-    )
-
-})
-
-
 test_that("model argument works with LDA", {
     skip_on_cran()
 
@@ -159,6 +162,53 @@ test_that("model argument works with LDA", {
     )
 })
 
+test_that("gamma is working", {
+
+    corp <- corpus_reshape(data_corpus_moviereviews[1:100])
+    toks <- tokens(corp,
+                   remove_punct = TRUE,
+                   remove_symbols = TRUE,
+                   remove_number = TRUE)
+    dfmt <- dfm(toks) %>%
+        dfm_remove(stopwords(), min_nchar = 2) %>%
+        dfm_trim(max_docfreq = 0.1, docfreq_type = "prop")
+
+    # make docid all unique
+    dfmt2 <- dfmt
+    dfmt2@docvars$docid_ <- dfmt2@docvars$docname_
+
+    set.seed(1234)
+    lda1 <- textmodel_lda(dfmt, k = 5, gamma = 0)
+
+    set.seed(1234)
+    expect_warning(
+        lda2 <- textmodel_lda(dfmt2, k = 5, gamma = 0.5),
+        "gamma has no effect when docid are all unique"
+    )
+
+    expect_equal(lda1$phi, lda2$phi)
+    expect_equal(lda1$theta, lda2$theta)
+
+    set.seed(1234)
+    lda3 <- textmodel_lda(dfmt, k = 5, gamma = 0.1)
+    expect_gt(mean(diff(as.integer(topics(lda3))) == 0, na.rm = TRUE),
+              mean(diff(as.integer(topics(lda2))) == 0, na.rm = TRUE))
+
+    set.seed(1234)
+    lda4 <- textmodel_lda(dfmt, k = 5, gamma = 0.2)
+    expect_gt(mean(diff(as.integer(topics(lda4))) == 0, na.rm = TRUE),
+              mean(diff(as.integer(topics(lda3))) == 0, na.rm = TRUE))
+
+    expect_error(
+        textmodel_lda(dfmt, k = 5, gamma = -0.1),
+        "The value of gamma must be between 0 and 1"
+    )
+
+    expect_error(
+        textmodel_lda(dfmt, k = 5, gamma = 2.0),
+        "The value of gamma must be between 0 and 1"
+    )
+})
 
 test_that("select and min_prob are working", {
 
@@ -218,7 +268,7 @@ test_that("select and min_prob are working", {
 
     expect_error(
         topics(lda, select = character()),
-        "The length of select must be between 2 and 5"
+        "The length of select must be between 1 and 5"
     )
 
     expect_error(
@@ -226,3 +276,4 @@ test_that("select and min_prob are working", {
         "Selected topics must be in the model"
     )
 })
+
