@@ -23,14 +23,14 @@ test_that("divergence() is working", {
     expect_equal(divergence(lda),
                  0.34, tolerance = 0.01)
 
-    expect_gt(divergence(lda, weighted = FALSE),
-              divergence(lda, weighted = TRUE))
+    expect_gt(divergence(lda, regularize = FALSE),
+              divergence(lda, regularize = TRUE))
 
     expect_gt(divergence(lda),
               divergence(lda, min_size = 0.1))
 
     expect_equal(divergence(slda),
-                 0.31, tolerance = 0.01)
+                 0.29, tolerance = 0.01)
 
     expect_silent(divergence(lda, select = c("topic1", "topic2")))
     expect_error(
@@ -46,8 +46,8 @@ test_that("divergence() is working", {
         "Selected topics must be in the model"
     )
 
-    expect_gt(divergence(slda, weighted = FALSE),
-              divergence(slda, weighted = TRUE))
+    expect_gt(divergence(slda, regularize = FALSE),
+              divergence(slda, regularize = TRUE))
 
     expect_gt(divergence(slda),
               divergence(slda, min_size = 0.1))
@@ -72,8 +72,10 @@ test_that("sizes() is working", {
     size1 <- sizes(lda)
     expect_true(is.numeric(size1))
     expect_equal(names(size1), c("topic1", "topic2", "topic3", "topic4", "topic5"))
+    expect_equal(sum(size1), 1)
 
     size2 <- sizes(slda)
     expect_true(is.numeric(size2))
     expect_equal(names(size2), c("romance", "sifi", "other"))
+    expect_equal(sum(size2), 1)
 })
